/*******************************************************************************
* application.cpp:                                                             *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*******************************************************************************/


//------------------------------------------------------------------------------
// Preamble
//------------------------------------------------------------------------------
#include <stddef.h>
#include <stdio.h>
#include <pthread.h>
#include <netdb.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/types.h>
#include <netinet/in.h>

#include "config.h"
#include "common.h"
#include "string.h"
#include "log.h"
#include "list.h"
#include "directory.h"
#include "ts.h"
#include "buffer.h"
#include "output.h"
#include "input.h"
#include "manager.h"
#include "admin.h"
#include "application.h"

#include "debug.h"



//------------------------------------------------------------------------------
// Global data shared by all the modules of the application
//------------------------------------------------------------------------------

// Logger
C_Log* pLog = NULL;



//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
C_Application::C_Application()
{
  pLogger = NULL;
  pManager = NULL;
  pAdministrator = NULL;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
C_Application::~C_Application()
{
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Application::Init(int iArgc, char** paArgv)
{
  // Options whose default value will be modified by cmd line or cfg file info
  // and that will be given as initialisation arguments to the relevant objects
  C_String strCfgFile(DFLT_CFGFILE);
  C_String strLogFile(DFLT_LOGFILE);
  
  // Startup message
  printf("%s\n", PGRM_INFO);

  // Check environnement
//  int iRc = CheckEnvir();
  int iRc = NO_ERR;                     // KLUDGE !!!!

  // Retrieve global configuration from cmd line and cfg file
  if(!iRc)
    iRc = GetConfig(iArgc, paArgv, &strCfgFile, &strLogFile);

  // Init the logger
  if(!iRc)
  {
    pLogger = new C_Log;
    pLog = pLogger;
    iRc = pLog->Init(strLogFile);
  }

  // Register the application object by the logger
  if(!iRc)
  {
    hLog = pLog->Register("Appli", LOG_DBGMSG | LOG_FILE | LOG_SCR );
    if (!hLog)
      iRc = GEN_ERR;
  }
  
  // Init the manager
  if(!iRc)
  {
    pManager = new C_Manager(hLog);
    iRc = pManager->Init(strCfgFile);
  }
  
  // Init the administrator
  if(!iRc)
  {
    pAdministrator = new C_Admin(hLog);
    iRc = pAdministrator->Init(strCfgFile);
  }
  
  return iRc;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Application::Run()
{
  // Run the manager
  int iRc = pManager->Run();

  // Run the administrator
  if(!iRc)
    iRc = pAdministrator->Run();

  return iRc;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Application::Exit()
{
  int iRc = NO_ERR;

  // Destroy the administrator if any
  if(pAdministrator)
  {
    iRc = pAdministrator->Destroy();
    delete pAdministrator;
  }
  
  // Destroy the manager if any
  if(pManager)
  {
    iRc |= pManager->Destroy();
    delete pManager;
  }

  // Destroy the logger if any
  if(pLogger)
  {
    iRc |= pLogger->End();
    delete pLogger;
  }
  
  return iRc;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Application::CheckEnvir()
{
  int iRc = 0;
  
  if(geteuid() != 0)
  {
    printf("VideoLAN Server must be run as root\n");
    iRc = GEN_ERR;
  }

  else if(!getprotobyname("tcp"))
  {
    printf("Unknown protocol: TCP\n");
    iRc = GEN_ERR;
  }

  else if(!getprotobyname("udp"))
  {
    printf("Unknown protocol: UDP\n");
    iRc = GEN_ERR;
  }

  return iRc;
}



//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
void C_Application::Usage(const C_String& strProgName)
{
#define HELP ""				\
"Usage : %s\n"				\
"[-c <cfg>]: Config file\n"

  printf(HELP, (const char*)strProgName);
}


  
//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Application::GetConfig(int iArgc, char** paArgv, C_String* pstrCfgFile,
                             C_String* pstrLogFile)
{
  int iRc = NO_ERR;
  int iOptArg;
  
  // Parse command line
  // First parse optional args
  opterr = 0;		// Force getopt not to generate an error message for '?'
  while((iOptArg = getopt(iArgc, paArgv, "c:h")) != EOF)
  {
    switch(iOptArg)
    {
      case 'c':
        (*pstrCfgFile) = optarg;
        break;
      case 'h':
      {
        // Display help message
        Usage(*paArgv);
        iRc = GEN_ERR;
        break;
      }
      default:
      {
        printf("Invalid '%c' option or missing parameter", optopt);
        // Display help message
        Usage(*paArgv);
        iRc = GEN_ERR;
      }
    }
  }
    
  // Now parse the mandatory ones
  if(iArgc - optind != 0)
  {
    printf("Invalid %d extra argument(s)", iArgc - optind);
    Usage(*paArgv);
    iRc = GEN_ERR;
  }
  
  // Get additional options from configuration file
  if(!iRc)
  {
    // TO DO ???
  }
  
  return iRc;
}

