/*******************************************************************************
* buffer.cpp: Buffers management                                               *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*******************************************************************************/


//------------------------------------------------------------------------------
// Preamble
//------------------------------------------------------------------------------
#include <stddef.h>
#include <stdio.h>

#include "config.h"
#include "common.h"
#include "string.h"
#include "list.h"
#include "directory.h"
#include "ts.h"
#include "buffer.h"

#include "debug.h"



//******************************************************************************
// C_Fifo class
//******************************************************************************
//
//******************************************************************************

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_Fifo::C_Fifo(unsigned int iBuffSize)
{
  iSize = iBuffSize;
  apBuff = new (C_TsPacket*)[iBuffSize];
  ASSERT(apBuff);

  iStored = 0;
  iWhereToPush = 0;
  iWhereToPop = 0;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_Fifo::~C_Fifo()
{
  ASSERT(apBuff);
  delete[] apBuff;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// Return BUFFER_FULL if the FIFO is full
//------------------------------------------------------------------------------
int C_Fifo::Push(C_TsPacket* pPacket)
{
  ASSERT(pPacket);
  int iRc = NO_ERR;

//  printf("Push: %d %d\n", iStored, iSize);

  if (iStored < iSize)
  {
    // Add the element
    *(apBuff + iWhereToPush) = pPacket;
    // Update the WhereToPush index
    if (++iWhereToPush > iSize-1)
    {
      // We must roll to the 1st element of the array
      iWhereToPush = 0;
    }
          
    // The buffer has grown
    iStored++;
  }
  else
  {
//    printf("FIFO full\n");
    iRc = BUFFER_FULL;
  }
  
  return iRc;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// Return the first pushed packet, or NULL if the FIFO is empty
//------------------------------------------------------------------------------
C_TsPacket* C_Fifo::Pop()
{
  C_TsPacket* pPacket = NULL;
  
  if (iStored > 0)
  {
    // Read the element
    pPacket = *(apBuff + iWhereToPop);

    // Update the WhereToPop pointer
    if (++iWhereToPop > iSize-1)
    {
      // We must roll to the 1st element of the array */
      iWhereToPop = 0;
    }

    // The buffer has shorten
    iStored--;
  }

//  printf("Pop: iStored is now %d\n", iStored);
  return pPacket;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_TsPacket* C_Fifo::operator [] (unsigned int iIndex)
{
  // Check that there is a valid packet stored at the given index
  ASSERT(iIndex < iStored);

  // Return the address of the packet
  unsigned int iElemPos = iWhereToPop + iIndex;

  if(iElemPos > iSize)
    iElemPos = iElemPos - iSize;
  
  return *(apBuff + iElemPos);
}



//******************************************************************************
// C_NetList class
//******************************************************************************
//
//******************************************************************************

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_NetList::C_NetList(unsigned int iSize = 2^8)
{
  iBuffSize = iSize;
  
  // Create the buffer of TS packets
  aPackets = new C_TsPacket[iSize];
  ASSERT(aPackets);

  // Create the buffer of pointers to free TS packets
  apFreePackets = new (C_TsPacket*)[iSize];
  ASSERT(apFreePackets);
  
  // Now init the second buffer
  for(unsigned int iIndex = 0; iIndex < iSize; iIndex++)
  {
    apFreePackets[iIndex] = &aPackets[iIndex];
  }
  iFirstFree = 0;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
C_NetList::~C_NetList()
{
  ASSERT(aPackets);
  delete[] aPackets;

  ASSERT(apFreePackets);
  delete[] apFreePackets;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
// Return the address of the fetched packet or NULL if the netlist is empty 
//------------------------------------------------------------------------------
C_TsPacket* C_NetList::GetPacket()
{
  C_TsPacket* pFreePacket = NULL;

  // Get the addree of the first free packet if any
  if(iFirstFree < iBuffSize)
  {
    pFreePacket = apFreePackets[iFirstFree];
    iFirstFree++;
  }

    return pFreePacket;
}


//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
void C_NetList::ReleasePacket(C_TsPacket* pPacket)
{
  ASSERT(pPacket);
  ASSERT(iFirstFree > 0 && iFirstFree < iBuffSize);    // Check everything is OK
  
  iFirstFree--;
  apFreePackets[iFirstFree] = pPacket;
}
