/*******************************************************************************
* application.cpp:                                                             *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*******************************************************************************/


//------------------------------------------------------------------------------
// Preamble
//------------------------------------------------------------------------------
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <netinet/in.h>

#include "amsApiTypes.h"                                              // AMS API

#include "config.h"
#include "common.h"
#include "string.h"
#include "log.h"
#include "cfgfile.h"
#include "list.h"
#include "thread.h"
#include "directory.h"
#include "ts.h"
#include "buffer.h"
#include "output.h"
#include "input.h"
#include "vod.h"
#include "satellite.h"
#include "manager.h"

#include "debug.h"
#include "list.cpp"



//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
C_Manager::C_Manager(handle hLogger)
{
  hLog = hLogger;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
C_Manager::~C_Manager()
{
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Manager::Init(const C_String& strCfgFile)
{
  C_CfgFile cCfgFile(hLog);

  // Open the cfgfile
  int iRc = cCfgFile.Open(strCfgFile);

  // Find the section where is stored the config
  if(!iRc)
  {
    iRc = cCfgFile.FindSection("Inputs");
    if(iRc)
      Log(hLog, LOG_ERROR, "Init failed: no 'Inputs' section found in cfg file");
  }
  
  // Parse the file
  C_String strVarName, strVarVal;
  while(!iRc)
  {
    // Read the current entry
    iRc = cCfgFile.Read(&strVarName, &strVarVal);

    // Check it is OK
    if(iRc == CFG_OK)
    {
      C_Input* pInput = NULL;
      
      // We wait for an input number as variable name, nothing else
      if(strspn(strVarName, "0123456789") != strVarName.Length())
      {
        // Skip the entry and go directly to the next one
        Log(hLog, LOG_NOTE, "Extra parameter %s in config file %s skipped",
            (const char*)strVarName, (const char*)strCfgFile);
        continue;
      }
       
      // Create the input for the given source
      else if(strVarVal == "Vod")
      {
        // Create the requested input
        pInput = new C_Vod;
        ASSERT(pInput);
      }
      else if(strVarVal == "Satellite")
      {
        // Create the requested input
        pInput = new C_Satellite;
        ASSERT(pInput);
      }
      else
      {
        // Skip the entry and go directly to the next one
        Log(hLog, LOG_NOTE, "Extra parameter %s in config file %s skipped",
            (const char*)strVarVal, (const char*)strCfgFile);
        continue;
      }

      // If an input has been created, init and register it
      if(pInput)
      {
        // First init it
        iRc = pInput->Init(strCfgFile, strVarVal);

        // If the input is OK, register it else abort
        if(!iRc)
          iRc = Register(pInput);
        else
          delete pInput;
      
        // Log input initialisation status
        if(!iRc)
          Log(hLog, LOG_NOTE, "%s source sucessfully initialised",
              (const char*)strVarVal);
        else
          Log(hLog, LOG_ERROR, "%s source initialisation failed",
              (const char*)strVarVal);
      }
    }
  }

  if(iRc == CFG_END)
    iRc = cCfgFile.Close();
  
  return iRc;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Manager::Run()
{
  int iRc = NO_ERR;
  unsigned int iIndex;
  
  // Run all the registered input sources
  for(iIndex = 0; iIndex < cInputList.Size() && !iRc; iIndex++)
  {
    iRc |= cInputList[iIndex].Run();

    if(iRc)
      Log(hLog, LOG_ERROR, "Error when launching input %d", iIndex);
    else
      Log(hLog, LOG_NOTE, "Input %d correctly launched", iIndex);
  }

  return iRc;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Manager::Destroy()
{
  int iRc = NO_ERR;
  unsigned int iIndex;
  
  // Destroy all input sources
  for(iIndex = 0; iIndex < cInputList.Size(); iIndex++)
  {
    iRc |= cInputList[iIndex].Destroy();
  }
  
  return iRc;
}


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
int C_Manager::Register(C_Input* pInput)
{
  ASSERT(pInput);

  int iRc = cInputList.PushEnd(pInput);
  
  return iRc;
}
